﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SHPSeperatorByGRID
{
    public partial class Form1 : Form
    {
        private Dictionary<int, int> savedRows = new Dictionary<int, int>();
        
        public Form1()
        {
            InitializeComponent();
        }

        private void btnSetLayer_Click(object sender, EventArgs e)
        {
            openFileDialog.Filter = "SHP Files(*.shp)|*.shp";
            DialogResult DR = openFileDialog.ShowDialog();
            if (DR == DialogResult.OK)
            {
                String FileName = openFileDialog.FileName;
                Xr.Layers.RemoveAllLayer();
                Xr.Layers.AddShapeMapLayer("lyr", FileName);
                Xr.WaitForAllConnections();
                Xr.ZoomFullExtent();
            }
        }

        private XrMapLib.FieldSetDefine GetFieldDefine(XrMapLib.IAttributeTable table)
        {
            XrMapLib.FieldSet fieldSet = table.FieldSet;

            XrMapLib.FieldSetDefine FSD = new XrMapLib.FieldSetDefine();
            int cntFields = fieldSet.FieldCount;
            for (int iField = 0; iField < cntFields; ++iField)
            {
                XrMapLib.XrFieldTypeEnum type = fieldSet.GetFieldType(iField);
                String fieldName = fieldSet.GetFieldName(iField);
                if (type == XrMapLib.XrFieldTypeEnum.XrDouble || type == XrMapLib.XrFieldTypeEnum.XrFloat)
                {
                    FSD.AddFloatField(fieldName, 10, 2);
                }
                else if (type == XrMapLib.XrFieldTypeEnum.XrInteger || type == XrMapLib.XrFieldTypeEnum.XrShort || type == XrMapLib.XrFieldTypeEnum.XrVeryShort)
                {
                    FSD.AddIntegerField(fieldName, 10);
                }
                else if (type == XrMapLib.XrFieldTypeEnum.XrString)
                {
                    FSD.AddStringField(fieldName, 25);
                }
            }

            FSD.AddIntegerField("IFID", 10);

            return FSD;
        }

        private XrMapLib.SpatialOperator GetGridOp(XrMapLib.Extent MBR)
        {
            XrMapLib.IPolygonGraphicElement gr = new XrMapLib.PolygonGraphicElement();

            XrMapLib.CoordinateListCollection clc = new XrMapLib.CoordinateListCollection();
            XrMapLib.CoordinateList cl = new XrMapLib.CoordinateList();

            cl.Add(MBR.MinX, MBR.MinY);
            cl.Add(MBR.MaxX, MBR.MinY);
            cl.Add(MBR.MaxX, MBR.MaxY);
            cl.Add(MBR.MinX, MBR.MaxY);
            //cl.Add(MBR.MinX, MBR.MinY);
            clc.Add(cl);

            gr.SetPolygonList(clc);   
            XrMapLib.GraphicElement grid = gr as XrMapLib.GraphicElement;

            return grid.ToSpatialOperator();
        }

        private XrMapLib.ValueList GetAttributeValueList(XrMapLib.IAttributeRow row, int fieldsCount)
        {
            XrMapLib.ValueList values = new XrMapLib.ValueList();

            for (int iField = 0; iField < fieldsCount; ++iField)
            {
                String value = row.GetValueAsString(iField);
                values.AddValueByString(value);
            }

            return values;
        }

        private XrMapLib.Coordinate GetShapeCoord(XrMapLib.IShapeRow row)
        {
            XrMapLib.Coordinate result = new XrMapLib.Coordinate();
            XrMapLib.Coordinate coord = row.GetVertex(0);
            result.X = coord.X;
            result.Y = coord.Y;

            return result;
        }

        private XrMapLib.CoordinateListCollection GetShapeCoordList(XrMapLib.IShapeRow row)
        {
            XrMapLib.CoordinateListCollection CLC = new XrMapLib.CoordinateListCollection();

            int cntParts = row.PartCount;
            for (int iPart = 0; iPart < cntParts; ++iPart)
            {
                int cntVertices = row.GetVertexCountFromPart(iPart);
                XrMapLib.CoordinateList CL = new XrMapLib.CoordinateList();  
                for (int iVertex = 0; iVertex < cntVertices; ++iVertex)
                {
                    XrMapLib.Coordinate coord = row.GetVertexFromPart(iPart, iVertex);
                    CL.Add(coord.X, coord.Y);
                }

                CLC.Add(CL);
            }

            return CLC;
        }

        private void btnRun_Click(object sender, EventArgs e)
        {
            Xr.StopMapDrawing();
            Xr.WaitForAllDrawing();

            savedRows.Clear();

            bool bExisted = Xr.Layers.IsLayerExist("lyr");
            if (!bExisted) return;

            XrMapLib.IShapeMapLayer lyr = Xr.Layers.GetLayerAsShapeMap("lyr");
            XrMapLib.Extent lyrMBR = lyr.MBR;
            int Rows = Int32.Parse(tbRows.Text);
            int Columns = Int32.Parse(tbColumns.Text);
            double GridWidth = (lyrMBR.MaxX - lyrMBR.MinX) / (double)Columns;
            double GridHeight = (lyrMBR.MaxY - lyrMBR.MinY) / (double)Rows;;

            XrMapLib.IShapeTable shpTbl = lyr.ShapeTable;
            XrMapLib.IAttributeTable attTbl = lyr.AttributeTable;
            XrMapLib.FieldSetDefine FSD = GetFieldDefine(attTbl);
            XrMapLib.Tool tool = new XrMapLib.Tool();
            int fieldsCount = attTbl.FieldSet.FieldCount;
            XrMapLib.ValueList VL;
            int cntRows = shpTbl.RowCount;
            XrMapLib.IShapeRow shp = null;
            bool bPointType = shpTbl.ShapeType == XrMapLib.XrShapeTypeEnum.XrPoint;

            for (int iRow = 0; iRow < Rows; ++iRow)
            {
                for (int iColumn = 0; iColumn < Columns; ++iColumn)
                {
                    String FileName = tbPrefixName.Text + "_" + iRow + "_" + iColumn + ".shp";

                    XrMapLib.EsriShpFileHandler SFH = tool.CreateEsriShpFileHandler(FileName, lyr.ShapeType, FSD);
                    if (SFH == null)
                    {
                        MessageBox.Show("Error: Creating shp file");
                        return;
                    }

                    XrMapLib.Extent GridMBR = new XrMapLib.Extent();
                    GridMBR.MinX = lyrMBR.MinX + GridWidth * (double)iColumn;
                    GridMBR.MinY = lyrMBR.MinY + GridHeight * (double)iRow;
                    GridMBR.MaxX = GridMBR.MinX + GridWidth;
                    GridMBR.MaxY = GridMBR.MinY + GridHeight;
                    XrMapLib.SpatialOperator gridOp = GetGridOp(GridMBR);

                    int cntAdded = 0;
                    for (int FID = 0; FID < cntRows; ++FID)
                    {
                        if (savedRows.ContainsKey(FID)) continue;

                        shp = shpTbl.GetRow(FID);
                        XrMapLib.IExtent rowMBR = shp.Extent;
                        if ((rowMBR.MinX <= GridMBR.MaxX) && (rowMBR.MaxX >= GridMBR.MinX) && (rowMBR.MinY <= GridMBR.MaxY) && (rowMBR.MaxY >= GridMBR.MinY))
                        {
                            if (shp.Load())
                            {
                                XrMapLib.SpatialOperator shapeOp = shp.ToSpatialOperator();
                                if (gridOp.Intersect(shapeOp))
                                {
                                    XrMapLib.IAttributeRow attr = attTbl.GetRow(FID);
                                    if (attr.Load())
                                    {
                                        VL = GetAttributeValueList(attr, fieldsCount);
                                        VL.AddValueByInteger(FID);

                                        if (bPointType)
                                        {
                                            XrMapLib.Coordinate COORD = GetShapeCoord(shp);
                                            bool bOK = SFH.AddPoint(COORD.X, COORD.Y, VL);
                                            if (!bOK)
                                            {
                                                MessageBox.Show("ERROR!");
                                            }
                                        }
                                        else
                                        {
                                            XrMapLib.CoordinateListCollection CLC = GetShapeCoordList(shp);
                                            bool bOK = SFH.AddMultiPart(CLC, VL);
                                            if (!bOK)
                                            {
                                                MessageBox.Show("ERROR!");
                                            }
                                        }

                                        cntAdded++;
                                        savedRows.Add(FID, 0);
                                        attr.Unload();
                                    }
                                }

                                shp.Unload();
                            }
                        }
                    }

                    SFH.Close();

                    if (cntAdded == 0)
                    {
                        String shpFileName = FileName.ToLower();
                        String dbfFileName = shpFileName.Replace(".shp", ".dbf");
                        String shxFileName = shpFileName.Replace(".shp", ".shx");

                        System.IO.File.Delete(shpFileName);
                        System.IO.File.Delete(dbfFileName);
                        System.IO.File.Delete(shxFileName);
                    }
                }
            }
        }
    }
}